﻿using System.Collections.Generic;
using System.Data;
using System.Web.UI;
using Vevo.Base.Data.Caching;
using Vevo.Base.Domain;
using Vevo.Domain;
using Vevo.Domain.Blogs;
using Vevo.Domain.DataInterfaces;
using Vevo.Shared.Utilities;

namespace Vevo.Data
{
    public class BlogRepository : IBlogRepository
    {
        #region Private

        private const string RepositoryName = "BlogRepository";

        private const string GetOneSignature = "GetOne";
        private const string GetBlogIDFromUrlNameSignature = "GetBlogIDFromUrlName";
        private const string GetIDByTitleSignature = "GetIDByTitle";
        private const string GetAllSignature = "GetAll";
        private const string SearchBlogSignature = "SearchBlog";
        private const string GetArchiveListSignature = "GetArchiveList";
        private const string GetRecentPostSignature = "GetRecentPost";
        private const string GetTableSchemaSignature = "GetTableSchema";
        private const string GetBlogListByDateSignature = "GetBlogListByDate";
        private const string GetBlogListByCategoryIDSignature = "GetBlogListByCategoryID";
        private const string GetQuickSearchSignature = "GetQuickSearch";

        private DataCache _dataCache = new DataCache( RepositoryName );
        private ICacheDependencyUtility _cacheDependency = new CacheDependencyUtility( new string[] { "Blog" } );

        private BlogRepositoryNoCache _repository = new BlogRepositoryNoCache();

        #endregion

        #region Constructors

        public BlogRepository()
        {
        }

        public BlogRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region Public Methods

        public Blog Save( Blog blog )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            Blog newBlog = _repository.Save( blog );
            _dataCache.ClearRepositoryCache();
            return newBlog;
        }

        public Blog GetOne( string blogID )
        {
            Blog result = (Blog) _dataCache.GetValue( GetOneSignature, _cacheDependency, blogID );

            if (result == null)
            {
                result = _repository.GetOne( blogID );
                _dataCache.SetValue( GetOneSignature, result, blogID );
            }

            return result;
        }

        public string GetBlogIDFromUrlName( string urlName )
        {
            object blogID = _dataCache.GetValue( GetBlogIDFromUrlNameSignature, _cacheDependency, urlName );

            if (blogID == null)
            {
                blogID = _repository.GetBlogIDFromUrlName( urlName );
                _dataCache.SetValue( GetBlogIDFromUrlNameSignature, blogID, urlName );
            }

            return blogID.ToString();
        }

        public string GetIDByTitle( string blogTitle )
        {
            object result = _dataCache.GetValue( GetIDByTitleSignature, _cacheDependency, blogTitle );

            if (result == null)
            {
                result = _repository.GetIDByTitle( blogTitle );
                _dataCache.SetValue( GetIDByTitleSignature, result, blogTitle );
            }

            return result.ToString();
        }

        public void Delete( string blogID )
        {
            _repository.Delete( blogID );
            _dataCache.ClearRepositoryCache();
        }

        public IList<Blog> GetAll( string sortBy )
        {
            IList<Blog> list = (IList<Blog>) _dataCache.GetValue( GetAllSignature, _cacheDependency, sortBy );

            if (list == null)
            {
                list = _repository.GetAll( sortBy );
                _dataCache.SetValue( GetAllSignature, sortBy );
            }

            return list;
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            IList<TableSchemaItem> list = (IList<TableSchemaItem>) _dataCache.GetValue( GetTableSchemaSignature, _cacheDependency );

            if (list == null)
            {
                list = _repository.GetTableSchema();
                _dataCache.SetValue( GetTableSchemaSignature, list );
            }

            return list;
        }

        public IList<Blog> SearchBlog(
            string sortBy,
            SearchFilter searchFilter,
            int startIndex,
            int endIndex,
            out int howManyItems,
            string storeID,
            bool allDate,
            BoolFilter isEnabled )
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                SearchBlogSignature,
                _cacheDependency,
                sortBy,
                searchFilter.ToString(),
                startIndex,
                endIndex,
                storeID,
                allDate,
                isEnabled );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.SearchBlog(
                    sortBy, searchFilter, startIndex, endIndex, out howManyItems, storeID, allDate, isEnabled );
                result.Second = howManyItems;
                _dataCache.SetValue(
                    SearchBlogSignature,
                    result,
                    sortBy,
                    searchFilter.ToString(),
                    startIndex,
                    endIndex,
                    storeID,
                    allDate,
                    isEnabled );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return (IList<Blog>) result.First;
        }

        public DataTable GetArchiveList( string storeID )
        {
            DataTable result = (DataTable) _dataCache.GetValue( GetArchiveListSignature, _cacheDependency, storeID );

            if (result == null)
            {
                result = _repository.GetArchiveList( storeID );
                _dataCache.SetValue( GetArchiveListSignature, result, storeID );
            }

            return result;
        }

        public DataTable GetArchiveList( string storeID, string limitRow )
        {
            DataTable result = (DataTable) _dataCache.GetValue( GetArchiveListSignature, _cacheDependency, storeID, limitRow );

            if (result == null)
            {
                result = _repository.GetArchiveList( storeID, limitRow );
                _dataCache.SetValue( GetArchiveListSignature, result, storeID, limitRow );
            }

            return result;
        }

        public IList<Blog> GetRecentPost( string storeID, string limitRow )
        {
            IList<Blog> list = (IList<Blog>) _dataCache.GetValue( GetRecentPostSignature, _cacheDependency, storeID, limitRow );

            if (list == null)
            {
                list = _repository.GetRecentPost( storeID, limitRow );
                _dataCache.SetValue( GetRecentPostSignature, list, storeID, limitRow );
            }

            return list;
        }

        public IList<Blog> GetBlogListByDate(
            string month,
            string year,
            int startIndex,
            int endIndex,
            out int howManyItems,
            string storeID )
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                GetBlogListByDateSignature,
                _cacheDependency,
                month,
                year,
                startIndex,
                endIndex,
                storeID );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.GetBlogListByDate(
                    month, year, startIndex, endIndex, out howManyItems, storeID );
                result.Second = howManyItems;
                _dataCache.SetValue(
                    GetBlogListByDateSignature,
                    result,
                    month,
                    year,
                    startIndex,
                    endIndex,
                    storeID );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return (IList<Blog>) result.First;
        }

        public IList<Blog> GetBlogListByCategoryID(
            string sortBy,
            string categoryID,
            int startIndex,
            int endIndex,
            out int howManyItems,
            string storeID,
            BoolFilter isEnabled )
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                GetBlogListByCategoryIDSignature,
                _cacheDependency,
                sortBy,
                categoryID,
                startIndex,
                endIndex,
                storeID,
                isEnabled.ToString() );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.GetBlogListByCategoryID(
                    sortBy, categoryID, startIndex, endIndex, out howManyItems, storeID, isEnabled );
                result.Second = howManyItems;
                _dataCache.SetValue(
                    GetBlogListByCategoryIDSignature,
                    result,
                    sortBy,
                    categoryID,
                    startIndex,
                    endIndex,
                    storeID,
                    isEnabled.ToString() );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return (IList<Blog>) result.First;
        }

        public int GetBlogCountByCategoryID( string categoryID, string storeID, BoolFilter isEnabled )
        {
            return _repository.GetBlogCountByCategoryID( categoryID, storeID, isEnabled );
        }

        public IList<Blog> QuickSearch(
            string searchTerm,
            string[] searchFieldList,
            int startIndex,
            int endIndex,
            out int howManyItems,
            string storeID
            )
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                GetQuickSearchSignature,
                _cacheDependency,
                searchTerm,
                string.Join( ",", searchFieldList ),
                startIndex,
                endIndex,
                storeID );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.QuickSearch(
                    searchTerm, searchFieldList, startIndex, endIndex, out howManyItems, storeID );
                result.Second = howManyItems;
                _dataCache.SetValue(
                    GetQuickSearchSignature,
                    result,
                    searchTerm,
                    string.Join( ",", searchFieldList ),
                    startIndex,
                    endIndex,
                    storeID );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return (IList<Blog>) result.First;
        }

        #endregion
    }
}
