using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Text;
using Vevo.Domain;
using Vevo.Domain.Base;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Products;
using Vevo.Shared.DataAccess;
using Vevo.Base.Data;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class OptionItemRepositoryNoCache : IOptionItemRepository
    {
        #region Private

        private const string FixedPriceTypeString = "Price";
        private const string PercentagePriceTypeString = "Percentage";


        private const string AllOptionItemFields =
            " OptionItem.OptionItemID, OptionItem.OptionGroupID, OptionItemType, PriceToAdd, " +
            "PercentageChange, WeightToAdd, ImageFile, Name ";

        private const string AllOptionItemLocaleFields = " OptionItemID, CultureID, Name ";


        private OptionItem.PriceType ConvertToPriceType( string optionItemType )
        {
            if (optionItemType == FixedPriceTypeString)
                return OptionItem.PriceType.Price;
            else
                return OptionItem.PriceType.Percentage;
        }

        private string ConvertFromPriceTypeToString( OptionItem.PriceType priceType )
        {
            if (priceType == OptionItem.PriceType.Price)
                return FixedPriceTypeString;
            else
                return PercentagePriceTypeString;
        }

        private OptionItem CreateOneFromRow( Culture culture, DataRow row )
        {
            OptionItem optionItem = new OptionItem( culture );

            optionItem.OptionItemID = row["OptionItemID"].ToString();
            optionItem.Name = row["Name"].ToString();
            optionItem.OptionGroupID = row["OptionGroupID"].ToString();
            optionItem.OptionItemType = ConvertToPriceType( row["OptionItemType"].ToString() );
            optionItem.PriceToAdd = decimal.Parse( row["PriceToAdd"].ToString() );
            optionItem.PercentageChange = double.Parse( row["PercentageChange"].ToString() );
            optionItem.WeightToAdd = double.Parse( row["WeightToAdd"].ToString() );
            optionItem.ImageFile = row["ImageFile"].ToString();

            return optionItem;
        }

        private void DeleteLocales( string optionItemID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM OptionItemLocale " +
                "WHERE OptionItemID = @OptionItemID; ",
                DataAccess.CreateParameterString( optionItemID ) );
        }

        private void InsertLocales( OptionItem optionItem )
        {
            foreach (OptionItemLocale locale in optionItem.Locales)
            {
                DataAccess.ExecuteNonQuery(
                    "INSERT INTO OptionItemLocale ( OptionItemID, CultureID, Name ) " +
                    "VALUES ( @OptionItemID, @CultureID, @Name )",
                    DataAccess.CreateParameterString( optionItem.OptionItemID ),
                    DataAccess.CreateParameterString( locale.CultureID ),
                    DataAccess.CreateParameterString( locale.Name ) );
            }
        }

        private OptionItem Create( OptionItem optionItem )
        {
            string newID = DataAccess.ExecuteScalar(
                "INSERT INTO OptionItem ( OptionGroupID, OptionItemType, PriceToAdd, " +
                "PercentageChange, WeightToAdd, ImageFile ) " +
                "VALUES ( @OptionGroupID, @OptionItemType, @PriceToAdd, @PercentageChange, " +
                "@WeightToAdd, @ImageFile);" +
                "SELECT @@Identity;",
                DataAccess.CreateParameterString( optionItem.OptionGroupID ),
                DataAccess.CreateParameterString( ConvertFromPriceTypeToString( optionItem.OptionItemType ) ),
                DataAccess.CreateParameterDecimalMoney( Convert.ToDecimal( optionItem.PriceToAdd ) ),
                DataAccess.CreateParameterDouble( optionItem.PercentageChange ),
                DataAccess.CreateParameterDouble( optionItem.WeightToAdd ),
                DataAccess.CreateParameterString( optionItem.ImageFile ) );

            optionItem.OptionItemID = newID;

            InsertLocales( optionItem );

            return optionItem;
        }

        private OptionItem Update( OptionItem optionItem )
        {
            DataAccess.ExecuteNonQuery(
                "UPDATE OptionItem " +
                "SET OptionGroupID = @OptionGroupID, OptionItemType = @OptionItemType, " +
                "PriceToAdd = @PriceToAdd, PercentageChange = @PercentageChange, " +
                "WeightToAdd = @weightToAdd, ImageFile = @ImageFile " +
                "WHERE OptionItemID = @OptionItemID ;",
                DataAccess.CreateParameterString( optionItem.OptionGroupID ),
                DataAccess.CreateParameterString( ConvertFromPriceTypeToString( optionItem.OptionItemType ) ),
                DataAccess.CreateParameterDecimalMoney( Convert.ToDecimal( optionItem.PriceToAdd ) ),
                DataAccess.CreateParameterDouble( Convert.ToDouble( optionItem.PercentageChange ) ),
                DataAccess.CreateParameterDouble( Convert.ToDouble( optionItem.WeightToAdd ) ),
                DataAccess.CreateParameterString( optionItem.ImageFile ),
                DataAccess.CreateParameterString( optionItem.OptionItemID ) );

            DeleteLocales( optionItem.OptionItemID );
            InsertLocales( optionItem );

            return optionItem;
        }

        private OptionItemLocale CreateOneLocaleFromRow( DataRow row )
        {
            OptionItemLocale locale = new OptionItemLocale();
            locale.CultureID = row["CultureID"].ToString();
            locale.Name = row["Name"].ToString();

            return locale;
        }

        private List<ILocale> ConvertLocaleTableToList( DataTable localeTable )
        {
            List<ILocale> list = new List<ILocale>();

            foreach (DataRow row in localeTable.Rows)
                list.Add( CreateOneLocaleFromRow( row ) );

            return list;
        }

        private IList<OptionItem> ConvertOptionItemTableToList( Culture culture, DataTable table )
        {
            IList<OptionItem> list = new List<OptionItem>();

            foreach (DataRow row in table.Rows)
                list.Add( CreateOneFromRow( culture, row ) );

            return list;
        }

        private string ConvertFieldName( string fieldName )
        {
            switch (fieldName.ToLower())
            {
                case "optionitemid":
                    return "OptionItem.OptionItemID";
                default:
                    return fieldName;
            }
        }

        private string CreateSortByText( string sortBy )
        {
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else if (sortBy.Trim().ToLower().StartsWith( "optionitemid" ))
                return "ORDER BY OptionItem." + sortBy.Trim() + " ";
            else
                return "ORDER BY " + sortBy + " ";
        }

        private IList<OptionItem> ConvertTableToList( Culture culture, DataTable table, int startIndex, int endIndex )
        {
            IList<OptionItem> list = new List<OptionItem>();

            for (int i = startIndex; i <= endIndex && i < table.Rows.Count; i++)
                list.Add( CreateOneFromRow( culture, table.Rows[i] ) );

            return list;
        }

        private void CopyOptionItemLocales( OptionItem optionItem, IList<ILocale> locales )
        {
            foreach (OptionItemLocale locale in locales)
            {
                optionItem.Locales.Add( locale );
            }
        }

        #endregion


        #region Public Methods

        public OptionItem Save( OptionItem optionItem )
        {
            OptionItem result;

            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                if (optionItem.OptionItemID == "0")
                    result = Create( optionItem );
                else
                    result = Update( optionItem );

                transaction.Complete();
            }

            return result;
        }

        public void Delete( string optionItemID )
        {
            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                DataAccess.ExecuteNonQuery(
                    "DELETE " +
                    "FROM OptionItem " +
                    "WHERE OptionItemID = @OptionItemID; ",
                    DataAccess.CreateParameterString( optionItemID ) );

                DeleteLocales( optionItemID ); 
                transaction.Complete();
            }
        }

        public void DeleteByOptionGroupID( string optionGroupID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT * FROM OptionItem WHERE OptionGroupID = @OptionGroupID;",
                DataAccess.CreateParameterString( optionGroupID ) );

            foreach (DataRow row in table.Rows)
            {
                Delete( row["OptionItemID"].ToString() );
            }
        }

        public void DeleteLocalesByCultureID( string cultureID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE " +
                "FROM OptionItemLocale " +
                "WHERE CultureID = @CultureID ;",
                DataAccess.CreateParameterString( cultureID ) );
        }

        public IList<ILocale> GetLocales( string optionItemID )
        {
            DataTable localeTable = DataAccess.ExecuteSelect(
                "SELECT " + AllOptionItemLocaleFields +
                "FROM OptionItemLocale " +
                "WHERE OptionItemID = @OptionItemID; ",
                DataAccess.CreateParameterString( optionItemID ) );

            return ConvertLocaleTableToList( localeTable );
        }

        public OptionItem GetOne( Culture culture, string optionItemID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllOptionItemFields +
                "FROM OptionItem LEFT JOIN " +
                "    (SELECT * FROM OptionItemLocale WHERE CultureID = @CultureID) AS L " +
                "ON OptionItem.OptionItemID = L.OptionItemID " +
                "WHERE OptionItem.OptionItemID = @OptionItemID; ",
                DataAccess.CreateParameterString( culture.CultureID ),
                DataAccess.CreateParameterString( optionItemID ) );

            if (table.Rows.Count > 0)
            {
                OptionItem optionItem = CreateOneFromRow( culture, table.Rows[0] );
                CopyOptionItemLocales( optionItem, GetLocales( optionItemID ) );
                return optionItem;
            }
            else
                return OptionItem.Null;
        }

        public IList<OptionItem> GetByOptionGroupID( Culture culture, string optionGroupID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllOptionItemFields +
                "FROM OptionItem LEFT JOIN " +
                "    (SELECT * FROM OptionItemLocale WHERE CultureID = @CultureID) AS L " +
                "ON OptionItem.OptionItemID = L.OptionItemID " +
                "WHERE OptionGroupID = @optionGroupID; ",
                DataAccess.CreateParameterString( culture.CultureID ),
                DataAccess.CreateParameterString( optionGroupID ) );

            return ConvertOptionItemTableToList( culture, table );
        }

        public IList<OptionItem> GetAllByOptionGroupID( Culture culture, string optionGroupID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT OptionItem.OptionItemID, OptionGroupID, OptionItemType, Name, PriceToAdd, " +
                "PercentageChange, WeightToAdd, ImageFile " +
                "FROM OptionItem LEFT JOIN " +
                "(SELECT * FROM OptionItemLocale WHERE CultureID = @CultureID) AS L " +
                "ON OptionItem.OptionItemID = L.OptionItemID " +
                "WHERE OptionGroupID = @optionGroupID ;",
                DataAccess.CreateParameterString( RepositoryUtilities.GetCultureID( culture ) ),
                DataAccess.CreateParameterString( optionGroupID ) );
            return ConvertTableToList( culture, table, 0, table.Rows.Count );
        }

        public IList<OptionItem> SearchOptionItem(
                Culture culture,
                string optionGroupID,
                string sortBy,
                SearchFilter searchfilter,
                int startIndex,
                int endIndex,
                out int howManyItems
            )
        {
            string sqlFieldName = ConvertFieldName( searchfilter.FieldName );
            ArrayList parameterList = new ArrayList();
            parameterList.Add( DataAccess.CreateParameterString( RepositoryUtilities.GetCultureID( culture ) ) );
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string commandSearch = searchFilterHelper.CreateCommandWithWhere( searchfilter, sqlFieldName, parameterList );
            parameterList.Add( DataAccess.CreateParameterString( optionGroupID ) );

            string query = "SELECT ROW_NUMBER() over ( " + CreateSortByText(sortBy) + ") as rownum, " +
                " * FROM (" +
                "SELECT OptionItem.OptionItemID, OptionGroupID, OptionItemType, Name, PriceToAdd, " +
                "PercentageChange, WeightToAdd, ImageFile " +
                "FROM OptionItem LEFT JOIN " +
                "(SELECT * FROM OptionItemLocale WHERE CultureID = @CultureID) AS L " +
                "ON OptionItem.OptionItemID = L.OptionItemID " +
                commandSearch +
                "AND OptionGroupID = @optionGroupID " +
                ") AS OptionItem ";

            string extQuery = "with SearchResult as (" + query + ") " +
                    " select * from SearchResult where rownum between " + (startIndex + 1).ToString() + " and " + (endIndex + 1).ToString() + ";";

            DataTable table = DataAccess.ExecuteSelect(extQuery,
            DataAccess.ConvertArrayListToDbParameters(parameterList));

            howManyItems = table.Rows.Count;

            //DataColumn[] keys = new DataColumn[] { table.Columns["OptionItem.OptionItemID"] };
            //table.PrimaryKey = keys;
            return ConvertTableToList(culture, table, startIndex, endIndex);
        }

        private int GetCountSearchOptionItem(
                Culture culture,
                string optionGroupID,
                string sortBy,
                SearchFilter searchfilter
            )
        {
            string sqlFieldName = ConvertFieldName(searchfilter.FieldName);
            ArrayList parameterList = new ArrayList();
            parameterList.Add(DataAccess.CreateParameterString(RepositoryUtilities.GetCultureID(culture)));
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            string commandSearch = searchFilterHelper.CreateCommandWithWhere(searchfilter, sqlFieldName, parameterList);
            parameterList.Add(DataAccess.CreateParameterString(optionGroupID));

            string query = "SELECT COUNT(*) AS Row FROM (" +
                "SELECT OptionItem.OptionItemID, OptionGroupID, OptionItemType, Name, PriceToAdd, " +
                "PercentageChange, WeightToAdd, ImageFile " +
                "FROM OptionItem LEFT JOIN " +
                "(SELECT * FROM OptionItemLocale WHERE CultureID = @CultureID) AS L " +
                "ON OptionItem.OptionItemID = L.OptionItemID " +
                commandSearch +
                "AND OptionGroupID = @optionGroupID " +
                CreateSortByText(sortBy) +
                ") AS Row";

            string countResult = DataAccess.ExecuteScalar(query,
            DataAccess.ConvertArrayListToDbParameters(parameterList));

            return Convert.ToInt32(countResult);
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT OptionItem.OptionItemID, OptionItemType, Name, PriceToAdd, " +
                "PercentageChange, WeightToAdd, ImageFile " +
                "FROM OptionItem LEFT JOIN " +
                "(SELECT * FROM OptionItemLocale) AS L " +
                "ON OptionItem.OptionItemID = L.OptionItemID " +
                "WHERE 1 = 0 ;" );

            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }

        #endregion
    }
}
