using System;
using System.Data;
using System.Collections;
using System.Collections.Generic;
using System.Configuration;

using Vevo;
using Vevo.Domain;
using Vevo.Domain.Configurations;
using Vevo.Domain.Discounts;
using Vevo.Domain.Orders;
using Vevo.Domain.Products;
using Vevo.Domain.Users;
using Vevo.Shared.Utilities;

namespace Vevo.Domain.Discounts
{
    /// <summary>
    /// Summary description for Discount
    /// </summary>
    public class QuantityDiscountCalculator
    {
        #region Private

        private decimal CalculateDiscount( string discountGroupID, int qty, decimal subtotal )
        {
            if (ConvertUtilities.ToInt32( discountGroupID ) == 0)
                return 0;

            DiscountGroup discountGroup = DataAccessContext.DiscountGroupRepository.GetOne( discountGroupID );
            return discountGroup.GetDiscount( qty, subtotal );
        }

        private decimal CalculateSeparatedOption(
            CartItemGroup cartItemGroup,
            WholesaleStatus wholesaleStatus,
            out IList<decimal> discountLines )
        {
            decimal result = 0;
            decimal unitPrice = 0;
            discountLines = new List<decimal>();

            foreach (ICartItem item in cartItemGroup.CartItems)
            {
                decimal discount = 0;
                if (!String.IsNullOrEmpty( item.DiscountGroupID ) && item.DiscountGroupID != "0")
                {
                    if (IsProductOptionDiscountable( item.DiscountGroupID ))
                        unitPrice = item.GetSubtotal( wholesaleStatus );
                    else
                    {
                        unitPrice = item.GetUnitPriceWithoutProductOption( wholesaleStatus ) * item.Quantity;
                    }


                    discount = CalculateDiscount(
                        item.DiscountGroupID,
                        item.Quantity,
                        unitPrice );

                    result += discount;

                }
                discountLines.Add( discount );
            }
            return result;
        }

        private bool CheckDiscountGroupID( OrderItem orderItem, out string discountGroupID )
        {
            Product product = DataAccessContext.ProductRepository.GetOne( Culture.Null, orderItem.ProductID, Stores.NullStore.Null.StoreID );

            if (String.IsNullOrEmpty( product.DiscountGroupID ) && product.DiscountGroupID == "0")
            {
                discountGroupID = "0";
                return false;
            }
            else
            {
                discountGroupID = product.DiscountGroupID;
                return true;
            }
        }

        private decimal CalculateUsingProductID(
            CartItemGroup cartItemGroup,
            WholesaleStatus wholesaleStatus,
            out IList<decimal> discountLines )
        {
            decimal result = 0;
            ArrayList itemCalculate = new ArrayList();
            bool productIDDuplicated = false;
            discountLines = new List<decimal>();

            foreach (ICartItem item in cartItemGroup.CartItems)
            {
                decimal discount = 0;
                if (!String.IsNullOrEmpty( item.DiscountGroupID ) && item.DiscountGroupID != "0")
                {
                    if (!SearchItemAlreadyCalculate( itemCalculate, item.ProductID ))
                    {
                        discount = CalculatePerProductID(
                            cartItemGroup.CartItems,
                            item.ProductID,
                            item.DiscountGroupID,
                            wholesaleStatus,
                            out productIDDuplicated );
                        result += discount;

                        if (productIDDuplicated)
                            itemCalculate.Add( item.ProductID );
                    }
                }
                discountLines.Add( discount );
            }
            return result;
        }

        private decimal CalculateFromRule(
            CartItemGroup cartItemGroup,
            WholesaleStatus wholesaleStatus,
            out IList<decimal> discountLines )
        {

            if (DataAccessContext.Configurations.GetBoolValue( "QuantityDiscountSeparateOption" ))
                return CalculateSeparatedOption( cartItemGroup, wholesaleStatus, out discountLines );
            else
                return CalculateUsingProductID( cartItemGroup, wholesaleStatus, out discountLines );

        }

        private bool SearchItemAlreadyCalculate( ArrayList productIDlist, string productID )
        {
            foreach (string item in productIDlist)
            {
                if (productID == item)
                    return true;
            }
            return false;
        }

        private decimal CalculatePerProductID(
            IList<ICartItem> cart,
            string productID,
            string discountGroupID,
            WholesaleStatus wholesaleStatus,
            out bool productIDDuplicated )
        {
            decimal result = 0;
            decimal unitPrice = 0;
            int quantity = 0;
            productIDDuplicated = false;
            foreach (ICartItem item in cart)
            {
                if (item.ProductID == productID)
                {
                    if (IsProductOptionDiscountable( item.DiscountGroupID ))
                        unitPrice = item.GetSubtotal( wholesaleStatus );
                    else
                        unitPrice = item.GetUnitPriceWithoutProductOption( wholesaleStatus ) * item.Quantity;

                    productIDDuplicated = true;
                    quantity += item.Quantity;
                    result += unitPrice;
                }
            }
            return CalculateDiscount( discountGroupID, quantity, result );
        }

        private bool IsProductOptionDiscountable( string discountGroupID )
        {
            DiscountGroup discountGroup = DataAccessContext.DiscountGroupRepository.GetOne( discountGroupID );
            if (discountGroup != null)
                return discountGroup.ProductOptionDiscount;
            else
                return true;
        }

        #endregion


        #region Constructors

        public QuantityDiscountCalculator()
        {
        }

        #endregion


        #region Public Methods

        public bool IsDiscount( string customerID )
        {
            if (DataAccessContext.Configurations.GetBoolValue( "IsDiscountAllCustomer" ))
                return true;
            else
                return false;
        }

        public decimal GetDiscount( CartItemGroup cartItemGroup, WholesaleStatus wholesaleStatus, out IList<decimal> discountLine )
        {
            return CalculateFromRule( cartItemGroup, wholesaleStatus, out discountLine );
        }

        #endregion

    }
}
