using System;
using System.Collections.Generic;
using System.Data;
using System.Configuration;

using Vevo.Domain.Configurations;
using Vevo.Domain.Discounts;
using Vevo.Domain.Orders;
using Vevo.Domain.Shipping;
using Vevo.Domain.Users;
using Vevo.Base.Domain;

namespace Vevo.Domain.Tax
{
    /// <summary>
    /// Summary description for TaxCalculator
    /// </summary>
    public class TaxCalculator
    {
        #region Private

        private decimal GetShippingCostAmountWithTax( decimal shippingHandling, Address shippingAddress )
        {
            if (DataAccessContext.Configurations.GetBoolValue( "IsTaxableShippingCost" )
                && DataAccessContext.Configurations.GetValue( "ShippingTaxClass" ) != "0")
                return shippingHandling
                    * (decimal) GetTaxRate( shippingAddress, DataAccessContext.Configurations.GetValue( "ShippingTaxClass" ) ) / 100.0m;
            else
                return 0;
        }

        private decimal GetDiscountableTaxAmount( CartItemGroup cartItemGroup, IList<decimal> discountLines, Address shippingAddress )
        {
            decimal result = 0;
            //int lineCount = 0;
            if (DataAccessContext.Configurations.GetBoolValue( "TaxPriceAfterDiscount" ))
            {
                for (int i = 0; i < cartItemGroup.CartItems.Count; i++)
                {
                    ICartItem cartItem = cartItemGroup.CartItems[i];
                    if (cartItem.IsTaxable)
                    {
                        if (discountLines != null && discountLines.Count > 0)
                        {
                            result += discountLines[i] * ((decimal) GetTaxRate( shippingAddress, cartItem.TaxClassID ) / 100.0m);
                        }
                    }
                    //lineCount++;
                }
            }
            return result;
        }

        private decimal GetDiscountableAmount( decimal discount )
        {
            if (DataAccessContext.Configurations.GetBoolValue( "TaxPriceAfterDiscount" ))
                return discount;
            else
                return 0;
        }

        //private decimal GetSubTotalTaxAmount( CartItemGroup cartItemGroup, Address shippingAddress, Customer customer )
        //{
        //    return GetSubTotalTaxAmount( cartItemGroup, shippingAddress, customer, false );
        //}

        private decimal GetSubTotalTaxAmount( CartItemGroup cartItemGroup, Address shippingAddress, Customer customer )
        {
            WholesaleStatus wholesaleStatus = customer.GetWholesaleStatus();

            decimal result = 0;

            foreach (ICartItem cartItem in cartItemGroup.CartItems)
            {
                if (cartItem.IsTaxable)
                {
                    result += cartItem.GetSubtotal( wholesaleStatus )
                        * ((decimal) GetTaxRate( shippingAddress, cartItem.TaxClassID ) / 100.0m);
                }
            }
            return result;
        }

        private decimal GetTaxableSubtotal( decimal subtotal )
        {
            if (!DataAccessContext.Configurations.GetBoolValue( "TaxIncludedInPrice" ))
                return subtotal;
            else
                return 0;
        }

        //private decimal GetNonLineItemTaxableAmount(
        //    decimal subtotal,
        //    decimal discount,
        //    decimal shippingCost,
        //    decimal handlingFee,
        //    Address shippingAddress )
        //{
        //    return GetShippingCostAmountWithTax( shippingCost + handlingFee, shippingAddress )
        //        - GetDiscountableAmount( discount )
        //        + GetTaxableSubtotal( subtotal );
        //}

        //private decimal GetNonLineItemTaxableAmount(
        //    CartItemGroup cartItemGroup,
        //    IList<decimal> discountLines,
        //    decimal shippingCost,
        //    decimal handlingFee,
        //    Address shippingAddress,
        //    Customer customer
        //    )
        //{
        //    return GetNonLineItemTaxableAmount( 
        //        cartItemGroup, discountLines, shippingCost, handlingFee, shippingAddress, customer, false );
        //}

        private decimal GetNonLineItemTaxableAmount(
            CartItemGroup cartItemGroup,
            IList<decimal> discountLines,
            decimal shippingCost,
            decimal handlingFee,
            Address shippingAddress,
            Customer customer )
        {
            return GetSubTotalTaxAmount( cartItemGroup, shippingAddress, customer )
                + GetShippingCostAmountWithTax( shippingCost + handlingFee, shippingAddress )
                - GetDiscountableTaxAmount( cartItemGroup, discountLines, shippingAddress );
        }

        private decimal GetTaxableSubtotal( CartItemGroup cartItemGroup, Customer customer )
        {
            WholesaleStatus wholesaleStatus = customer.GetWholesaleStatus( DataAccessContext.Configurations );

            decimal taxableSubtotal = 0;
            foreach (ICartItem cartItem in cartItemGroup.CartItems)
            {
                if (cartItem.TaxClassID != "0")
                    taxableSubtotal += cartItem.GetSubtotal( wholesaleStatus );
            }
            return taxableSubtotal;
        }

        private decimal GetTotalLineItemTax( Customer customer, CartItemGroup cartItemGroup )
        {
            decimal lineItemTax = 0;
            foreach (ICartItem cartItem in cartItemGroup.CartItems)
            {
                if (cartItem.TaxClassID != "0")
                {
                    lineItemTax +=
                        cartItem.GetCheckoutIncludedTaxAmount( customer.GetWholesaleStatus( DataAccessContext.Configurations ) );
                }
            }
            return lineItemTax;
        }

        #endregion


        #region Constructors

        public TaxCalculator()
        {
        }

        #endregion


        #region Public Methods

        public decimal GetTax(
            CartItemGroup cartItemGroup,
            CheckoutDetails checkoutDetails,
            Customer customer,
            IList<decimal> discountLines,
            decimal shippingCost,
            decimal handlingFee
            )
        {
            if (checkoutDetails.IsTaxExempt)
            {
                return 0;
            }

            if (customer.IsWholesale
                && !DataAccessContext.Configurations.GetBoolValue( "IsTaxableWholesaler" ))
            {
                return 0;
            }
            else
            {
                return GetNonLineItemTaxableAmount(
                    cartItemGroup, discountLines, shippingCost, handlingFee, checkoutDetails.ShippingAddress, customer );
            }
        }

        public double GetTaxRate( Address shippingAddress, string taxClassID )
        {
            if (taxClassID == "0")
                return 0;

            TaxClass taxClass = DataAccessContext.TaxClassRepository.GetOne( taxClassID );

            return taxClass.GetTaxRate( shippingAddress );
        }

        #endregion

    }
}
