using System;
using System.Collections.Generic;
using Vevo.Domain;
using Vevo.Domain.Users;
using Vevo.QuickBooks.Qbxml70;

namespace Vevo.QuickBooks.States
{
    /// <summary>
    /// Summary description for CustomerAddRequestState
    /// </summary>
    public class CustomerAddRequestState : QBWCState
    {
        private void MoveToNextState()
        {
            QBWCSession.CurrentState = new CustomerAddResponseState( QBWCSession );
        }

        private void MoveToProductVerifyRqState()
        {
            QBWCSession.CurrentState = new ProductVerifyRequestState( QBWCSession );
        }

        private IList<Customer> GetAddData()
        {
            IList<Customer> table = DataAccessContext.CustomerRepository.GetByIsQBExported( false, String.Empty );
            return table;
        }

        private string GetCustomerName( Customer customer )
        {
            string name;
            if ( DataAccessContext.Configurations.GetValue( "QBCustomerNameFormat" ).Equals( "lastfirst" ) )
                name = customer.BillingAddress.LastName + ", " + customer.BillingAddress.FirstName;
            else
                name = customer.BillingAddress.FirstName + " " + customer.BillingAddress.LastName;

            name =  QBUtilities.LimitLength( name, QBConst.CustomerNameMaxLength );
            return CreateUniqueName( name );
        }

        private CustomerAddRqType CreateOneRequest( Customer customer )
        {
            string customerName = GetCustomerName( customer );
            CustomerAddRqType req = new CustomerAddRqType();
            req.CustomerAdd = new CustomerAdd();
            
            req.CustomerAdd.Name = customerName;
            req.CustomerAdd.FirstName = QBUtilities.LimitLength( 
                customer.BillingAddress.FirstName, 
                QBConst.CustomerFirstNameMaxLength );
            req.CustomerAdd.LastName = QBUtilities.LimitLength( 
                customer.BillingAddress.LastName, 
                QBConst.CustomerLastNameMaxLength );
            
            req.CustomerAdd.BillAddress = new BillAddress();
            req.CustomerAdd.BillAddress.Addr1 = QBUtilities.LimitLength( 
                customer.BillingAddress.Address1, 
                QBConst.AddressLineMaxLength );
            req.CustomerAdd.BillAddress.Addr2 = QBUtilities.LimitLength( 
                customer.BillingAddress.Address2, 
                QBConst.AddressLineMaxLength );
            req.CustomerAdd.BillAddress.City = QBUtilities.LimitLength( customer.BillingAddress.City, QBConst.CityMaxLength );
            req.CustomerAdd.BillAddress.State = QBUtilities.LimitLength( customer.BillingAddress.State, QBConst.StateMaxLength );
            req.CustomerAdd.BillAddress.PostalCode = QBUtilities.LimitLength( customer.BillingAddress.Zip, QBConst.ZipMaxLength );
            req.CustomerAdd.BillAddress.Country = QBUtilities.LimitLength( customer.BillingAddress.Country, QBConst.CountryMaxLength );

            req.CustomerAdd.ShipAddress = new ShipAddress();
            req.CustomerAdd.ShipAddress.Addr1 = QBUtilities.LimitLength( 
                customer.ShippingAddress.Address1, 
                QBConst.AddressLineMaxLength );
            req.CustomerAdd.ShipAddress.Addr2 = QBUtilities.LimitLength(
                customer.ShippingAddress.Address2, 
                QBConst.AddressLineMaxLength );
            req.CustomerAdd.ShipAddress.City = QBUtilities.LimitLength( customer.ShippingAddress.City, QBConst.CityMaxLength );
            req.CustomerAdd.ShipAddress.State = QBUtilities.LimitLength( customer.ShippingAddress.State, QBConst.StateMaxLength );
            req.CustomerAdd.ShipAddress.PostalCode = QBUtilities.LimitLength( customer.ShippingAddress.Zip, QBConst.ZipMaxLength );
            req.CustomerAdd.ShipAddress.Country = QBUtilities.LimitLength( customer.ShippingAddress.Country, QBConst.CountryMaxLength );
            req.CustomerAdd.ShipAddress.Note = QBUtilities.LimitLength(
                    "Tel: " + customer.ShippingAddress.Phone +
                    " Fax: " + customer.ShippingAddress.Fax,
                    QBConst.ShipNoteMaxLength );

            req.CustomerAdd.Phone = QBUtilities.LimitLength( customer.BillingAddress.Phone, QBConst.PhoneMaxLength );
            req.CustomerAdd.Fax = QBUtilities.LimitLength( customer.BillingAddress.Fax, QBConst.FaxMaxLength );
            req.CustomerAdd.Email = customer.Email;
            req.CustomerAdd.Notes = QBUtilities.LimitLength(
                "CustomerID: " + customer.CustomerID +
                ", User Name: " + customer.UserName +
                ", Merchant Notes: " + customer.MerchantNotes,
                QBConst.CustomerNoteMaxLength );

            SalesTaxCodeRef tax = new SalesTaxCodeRef();
            if ( customer.IsTaxExempt )
                tax.FullName = "Non";
            else
                tax.FullName = "Tax";

            req.CustomerAdd.SalesTaxCodeRef = tax;
            
            QBWCSession.AddQBFullNameList( customerName, String.Empty );

            return req;
        }

        private CustomerAddRqType CreateAnonymousRequest()
        {
            CustomerAddRqType req = new CustomerAddRqType();
            req.CustomerAdd = new CustomerAdd();
            req.CustomerAdd.Name = CreateUniqueName( "Online Anonymous" );
            req.CustomerAdd.Notes = "CustomerID: 0";
            return req;
        }

        private string CreateCustomerAdd( out bool xmlReady )
        {
            IList<Customer> table = GetAddData();
            int lenght = table.Count;
            if ( DataAccessContext.Configurations.GetValue( "QBAnonymousListID" ).Trim() == "")
                lenght++;

            if (lenght == 0)
            {
                xmlReady = false;
                MoveToProductVerifyRqState();
                QBWCSession.ClearList();
                return String.Empty;
            }

            // Write QBXML
            QBXML qbxml = CreateRequestQbxml();

            object[] requests = new object[lenght];
            int index = 0;
            foreach (Customer customer in table)
            {
                requests[index] = CreateOneRequest( customer );
                index++;
            }

            if ( DataAccessContext.Configurations.GetValue( "QBAnonymousListID" ).Trim() == "")
                requests[index] = CreateAnonymousRequest();

            ((QBXMLMsgsRq) qbxml.Items[0]).Items = requests;

            xmlReady = true;
            string xml = CreateRequestXml( qbxml );

            return xml;
        }

        public CustomerAddRequestState( QBWCSession session )
            : base( session )
        {
        }

        public override string OnSendRequestXml( out bool xmlReady )
        {
            MoveToNextState();

            return CreateCustomerAdd( out xmlReady );
        }

    }
}
